/* debug.c -- the debug stuff
 * Copyright (C) 1996-2000 authors
 * This file is part of the xpk package.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 */

/* Written by Dirk Stcker <stoecker@amigaworld.com>
 * UNIX version by Vesa Halttunen <vesuri@jormas.com>
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdarg.h>
#include "xpk/xpk.h"
#include "xpkmaster.h"

typedef void (*putchtype) ();

#define FLAG_ERROR	(1<<0)
#define FLAG_RUNTIME	(1<<1)
#define FLAG_TAGLIST	(1<<2)
#define FLAG_FILE	(1<<3)

static unsigned int getFlags( void )
{
    unsigned int flags = FLAG_ERROR | FLAG_RUNTIME | FLAG_TAGLIST | FLAG_FILE;
    char *mode;

    if ((mode = getenv("XPKDEBUG"))) {
        flags = 0;
        for ( ; *mode != '\0'; mode++ ) {
            switch ( *mode ) {
                case 'F':
                    flags |= FLAG_FILE;
                    break;
                case 'E':
                    flags |= FLAG_ERROR;
                    break;
                case 'R':
                    flags |= FLAG_RUNTIME;
                    break;
                case 'T':
                    flags |= FLAG_TAGLIST;
                    break;
                default:
                    break;
            }
        }
    }

    return flags;
}

static FILE *getFH( unsigned int flags )
{
    if ( flags & FLAG_FILE ) {
        FILE *fh;
        if ((fh = fopen("/tmp/XpkMasterOut", "a+"))) {
            fseek(fh, 0, SEEK_END);
            return fh;
        }
    } else {
        return fopen( "/dev/stderr", "a+" );
    }

    return NULL;
}

void DebugTagList(char *msg, struct TagItem *taglist)
{
    unsigned int flags;
    flags = getFlags();

    if ( ( flags & FLAG_TAGLIST ) == 0 ) return;

    FILE *fh = getFH( flags );
    if ( ! fh ) return;
    
    fprintf(fh, "XpkM(%d):", getpid());

    fprintf(fh, msg);

    struct TagItem *ti;
    while((ti = NextTagItem(&taglist))) {
        unsigned int dmode = 0;
        const char *s;
         
        /* dmode == 1 - BOOL data, dmode == 2 - unknown */
        switch(ti->ti_Tag) {
            case XPK_InName:	  s = "XPK_InName, \"%s\" ($%08lx)"; break;
            case XPK_InFH:	  s = "XPK_InFH, $%08lx"; break;
            case XPK_InBuf:	  s = "XPK_InBuf, $%08lx"; break;
            case XPK_InHook:	  s = "XPK_InHook, $%08lx"; break;
            case XPK_OutName:	  s = "XPK_OutName, \"%s\" ($%08lx)"; break;
            case XPK_OutFH:	  s = "XPK_OutFH, %08lx"; break;
            case XPK_OutBuf:	  s = "XPK_OutBuf, $%08lx"; break;
            case XPK_GetOutBuf:	  s = "XPK_GetOutBuf, $%08lx"; break;
            case XPK_OutHook:	  s = "XPK_OutHook, $%08lx"; break;
            case XPK_InLen:	  s = "XPK_InLen, %lu"; break;
            case XPK_OutBufLen:	  s = "XPK_OutBufLen, %lu"; break;
            case XPK_GetOutLen:	  s = "XPK_GetOutLen, $%08lx"; break;
            case XPK_GetOutBufLen:  s = "XPK_GetOutBufLen, $%08lx"; break;
            case XPK_Password:	  s = "XPK_Password, \"%s\" ($%08lx)"; break;
            case XPK_GetError:	  s = "XPK_GetError, $%08lx"; break;
            case XPK_OutMemType:	  s = "XPK_OutMemType, $%08lx"; break;
            case XPK_PassThru:	  s = "XPK_PassThru, %s"; dmode = 1; break;
            case XPK_StepDown:	  s = "XPK_StepDown, %s"; dmode = 1; break;
            case XPK_ChunkHook:	  s = "XPK_ChunkHook, $%08ld"; break;
            case XPK_PackMethod:	  s = "XPK_PackMethod, \"%s\" ($%08lx)"; break;
            case XPK_ChunkSize:	  s = "XPK_ChunkSize, %lu"; break;
            case XPK_PackMode:	  s = "XPK_PackMode, %lu"; break;
            case XPK_NoClobber:	  s = "XPK_NoClobber, %s"; dmode = 1; break;
            case XPK_Ignore:	  s = "XPK_Ignore"; break;
            case XPK_TaskPri:	  s = "XPK_TaskPri, $ld"; break;
            case XPK_FileName:	  s = "XPK_FileName, \"%s\" ($%08lx)"; break;
            case XPK_ShortError:	  s = "XPK_ShortError, %s"; dmode = 1; break;
            case XPK_PackersQuery:  s = "XPK_PackersQuery, $%08lx"; break;
            case XPK_PackerQuery:	  s = "XPK_PackerQuery, $%08lx"; break;
            case XPK_ModeQuery:     s = "XPK_ModeQuery, $%08lx"; break;
            case XPK_LossyOK:	  s = "XPK_LossyOK, %s"; dmode = 1; break;
            case XPK_NoCRC:	  s = "XPK_NoCRC, $%08lx"; break;
            case XPK_Key16:	  s = "XPK_Key32, $%04lx"; break;
            case XPK_Key32:	  s = "XPK_Key32, $%08lx"; break;
            case XPK_NeedSeek:	  s = "XPK_NeedSeek, %s"; dmode = 1; break;
            case XPK_UseXfdMaster:  s = "XPK_UseXfdMaster, %s"; dmode = 1; break;
            case XPK_UseExternals:  s = "XPK_UseExternals, %s"; dmode = 1; break;
            case XPK_PassRequest:   s = "XPK_PassRequest, %s"; dmode = 1; break;
            case XPK_Preferences:	  s = "XPK_Preferences, %s"; dmode = 1; break;
            case XPK_ChunkReport:	  s = "XPK_ChunkReport, %s"; dmode = 1; break;
            case XPK_PassChars:	  s = "XPK_PassChars, $%08lx"; break;
            case XPK_PasswordBuf:   s = "XPK_PasswordBuf, $%08lx"; break;
            case XPK_PassBufSize:	  s = "XPK_PassBufSize, %lu"; break;
            case XPK_Key16BitPtr:	  s = "XPK_Key16BitPtr, $%08lx"; break;
            case XPK_Key32BitPtr:	  s = "XPK_Key32BitPtr, $%08lx"; break;
            case XPK_PubScreen:	  s = "XPK_PubScreen, $%08lx"; break;
            case XPK_PassTitle:	  s = "XPK_PassTitle, \"%s\" ($%08lx)"; break;
            case XPK_TimeOut:	  s = "XPK_TimeOut, %lu"; break;
            case XPK_PassWinLeft:	  s = "XPK_PassWinLeft, %lu"; break;
            case XPK_PassWinTop:	  s = "XPK_PassWinTop, %lu"; break;
            case XPK_PassWinWidth:  s = "XPK_PassWinWidth, %lu"; break;
            case XPK_PassWinHeight: s = "XPK_PassWinHeight, %lu"; break;
            case XPK_PassCenter:	  s = "XPK_PassCenter, %s"; dmode = 1; break;
            case XPK_PassVerify:	  s = "XPK_PassVerify, %s"; dmode = 1; break;
            default:		  s = "$%08lx, $%08lx"; dmode = 2; break;
        }

        fprintf(fh, "   ");

        if(dmode == 1) {
            const char *i = ti->ti_Data_u32 ? "TRUE" : "FALSE";
            fprintf(fh, s, i);
        }else if(dmode == 2) {
            unsigned int i0 = ti->ti_Tag;
            unsigned int i1 = (unsigned long)ti->ti_Data;
            fprintf(fh, s, i0, i1);
        } else {
            const char *s0 = (const char*)ti->ti_Data;
            unsigned int i1 = (unsigned long)ti->ti_Data;
            fprintf(fh, s, s0, i1);
        }

        fprintf(fh, "\n");
    }
    fprintf(fh, "   TAG_DONE\n");

    fprintf(fh, "\n");

    fclose( fh );
}

void DebugError(char *format, ...)
{
    unsigned int flags;
    flags = getFlags();

    if ( ( flags & FLAG_ERROR ) == 0 ) return;

    FILE *fh = getFH( flags );
    if ( ! fh ) return;
    
    fprintf(fh, "XpkM(%d):", getpid());

    va_list ap;
    va_start(ap, format);
    vfprintf(fh, format, ap);
    va_end(ap);

    fprintf(fh, "\n");

    fclose( fh );
}

void DebugRunTime(char *format, ...)
{
    unsigned int flags;
    flags = getFlags();

    if ( ( flags & FLAG_RUNTIME ) == 0 ) return;

    FILE *fh = getFH( flags );
    if ( ! fh ) return;
    
    fprintf(fh, "XpkM(%d):", getpid());

    va_list ap;
    va_start(ap, format);
    vfprintf(fh, format, ap);
    va_end(ap);

    fprintf(fh, "\n");

    fclose( fh );
}
